<?php
/*
+--------------------------------------------------------------------+
| CiviCRM version 1.7                                                |
+--------------------------------------------------------------------+
| Copyright CiviCRM LLC (c) 2004-2007                                |
+--------------------------------------------------------------------+
| This file is a part of CiviCRM.                                    |
|                                                                    |
| CiviCRM is free software; you can copy, modify, and distribute it  |
| under the terms of the Affero General Public License Version 1,    |
| March 2002.                                                        |
|                                                                    |
| CiviCRM is distributed in the hope that it will be useful, but     |
| WITHOUT ANY WARRANTY; without even the implied warranty of         |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.               |
| See the Affero General Public License for more details.            |
|                                                                    |
| You should have received a copy of the Affero General Public       |
| License along with this program; if not, contact CiviCRM LLC       |
| at info[AT]civicrm[DOT]org.  If you have questions about the       |
| Affero General Public License or the licensing  of CiviCRM,        |
| see the CiviCRM license FAQ at http://civicrm.org/licensing        |
+--------------------------------------------------------------------+
*/
/**
 *
 * @package CRM
 * @copyright CiviCRM LLC (c) 2004-2007
 * $Id$
 *
 */
$GLOBALS['_CRM_EVENT_DAO_EVENT']['_tableName'] =  'civicrm_event';
$GLOBALS['_CRM_EVENT_DAO_EVENT']['_fields'] =  null;
$GLOBALS['_CRM_EVENT_DAO_EVENT']['_links'] =  null;
$GLOBALS['_CRM_EVENT_DAO_EVENT']['_import'] =  null;
$GLOBALS['_CRM_EVENT_DAO_EVENT']['_export'] =  null;
$GLOBALS['_CRM_EVENT_DAO_EVENT']['_log'] =  false;

require_once 'CRM/Core/DAO.php';
require_once 'CRM/Utils/Type.php';
class CRM_Event_DAO_Event extends CRM_Core_DAO
{
    /**
     * static instance to hold the table name
     *
     * @var string
     * @static
     */
    
    /**
     * static instance to hold the field values
     *
     * @var array
     * @static
     */
    
    /**
     * static instance to hold the FK relationships
     *
     * @var string
     * @static
     */
    
    /**
     * static instance to hold the values that can
     * be imported / apu
     *
     * @var array
     * @static
     */
    
    /**
     * static instance to hold the values that can
     * be exported / apu
     *
     * @var array
     * @static
     */
    
    /**
     * static value to see if we should log any modifications to
     * this table in the civicrm_log table
     *
     * @var boolean
     * @static
     */
    
    /**
     * Event
     *
     * @var int unsigned
     */
    var $id;
    /**
     * Event belongs to which Domain?
     *
     * @var int unsigned
     */
    var $domain_id;
    /**
     * Event Title (e.g. Fall Fundraiser Dinner)
     *
     * @var string
     */
    var $title;
    /**
     * Brief summary of event. Text and html allowed. Displayed on Event Registration form and can be used on other CMS pages which need an event summary.
     *
     * @var text
     */
    var $summary;
    /**
     * Full description of event. Text and html allowed. Displayed on built-in Event Information screens.
     *
     * @var text
     */
    var $description;
    /**
     * Event Type ID.Implicit FK to civicrm_option_value where option_group = event_type.
     *
     * @var int unsigned
     */
    var $event_type_id;
    /**
     * Public events will be included in the iCal feeds. Access to private event information may be limited using ACLs.
     *
     * @var boolean
     */
    var $is_public;
    /**
     * Date and time that event starts.
     *
     * @var datetime
     */
    var $start_date;
    /**
     * Date and time that event ends. May be NULL if no defined end date/time
     *
     * @var datetime
     */
    var $end_date;
    /**
     * If true, include registration link on Event Info page.
     *
     * @var boolean
     */
    var $is_online_registration;
    /**
     * Text for link to Event Registration form which is displayed on Event Information screen when is_online_registration is true.
     *
     * @var string
     */
    var $registration_link_text;
    /**
     * Maximum number of registered participants to allow. After max is reached, a custom Event Full message is displayed. If NULL, allow unlimited number of participants.
     *
     * @var int unsigned
     */
    var $max_participants;
    /**
     * Message to display on Event Information page and INSTEAD OF Event Registration form if maximum participants are signed up. Can include email address/info about getting on a waiting list, etc. Text and html allowed.
     *
     * @var text
     */
    var $event_full_text;
    /**
     * Is this a PAID event? If true, one or more fee amounts must be set and a Payment Processor must be configured for Online Event Registration.
     *
     * @var boolean
     */
    var $is_monetary;
    /**
     * Contribution type assigned to paid event registrations for this event. Required if is_monetary is true.
     *
     * @var int unsigned
     */
    var $contribution_type_id;
    /**
     * Include a map block on the Event Information page when geocode info is available and a mapping provider has been specified?
     *
     * @var boolean
     */
    var $is_map;
    /**
     * Is this Event enabled or disabled/cancelled?
     *
     * @var boolean
     */
    var $is_active;
    /**
     * class constructor
     *
     * @access public
     * @return civicrm_event
     */
    function CRM_Event_DAO_Event() 
    {
        parent::CRM_Core_DAO();
    }
    /**
     * return foreign links
     *
     * @access public
     * @return array
     */
    function &links() 
    {
        if (!($GLOBALS['_CRM_EVENT_DAO_EVENT']['_links'])) {
            $GLOBALS['_CRM_EVENT_DAO_EVENT']['_links'] = array(
                'domain_id' => 'civicrm_domain:id',
            );
        }
        return $GLOBALS['_CRM_EVENT_DAO_EVENT']['_links'];
    }
    /**
     * returns all the column names of this table
     *
     * @access public
     * @return array
     */
    function &fields() 
    {
        if (!($GLOBALS['_CRM_EVENT_DAO_EVENT']['_fields'])) {
            $GLOBALS['_CRM_EVENT_DAO_EVENT']['_fields'] = array(
                'id' => array(
                    'name' => 'id',
                    'type' => CRM_UTILS_TYPE_T_INT,
                    'required' => true,
                ) ,
                'domain_id' => array(
                    'name' => 'domain_id',
                    'type' => CRM_UTILS_TYPE_T_INT,
                    'required' => true,
                ) ,
                'event_title' => array(
                    'name' => 'title',
                    'type' => CRM_UTILS_TYPE_T_STRING,
                    'title' => ts('Event Title') ,
                    'maxlength' => 255,
                    'size' => CRM_UTILS_TYPE_HUGE,
                    'import' => true,
                    'where' => 'civicrm_event.title',
                    'headerPattern' => '/^title$/i',
                    'dataPattern' => '',
                    'export' => true,
                ) ,
                'summary' => array(
                    'name' => 'summary',
                    'type' => CRM_UTILS_TYPE_T_TEXT,
                    'title' => ts('Event Summary') ,
                ) ,
                'event_description' => array(
                    'name' => 'description',
                    'type' => CRM_UTILS_TYPE_T_TEXT,
                    'title' => ts('Event Description') ,
                ) ,
                'event_type_id' => array(
                    'name' => 'event_type_id',
                    'type' => CRM_UTILS_TYPE_T_INT,
                    'title' => ts('Event Type ID') ,
                ) ,
                'is_public' => array(
                    'name' => 'is_public',
                    'type' => CRM_UTILS_TYPE_T_BOOLEAN,
                    'title' => ts('Is Event Public') ,
                ) ,
                'event_start_date' => array(
                    'name' => 'start_date',
                    'type' => CRM_UTILS_TYPE_T_DATE+CRM_UTILS_TYPE_T_TIME,
                    'title' => ts('Event Start Date') ,
                    'import' => true,
                    'where' => 'civicrm_event.start_date',
                    'headerPattern' => '/^start|(s(tart\s)?date)$/i',
                    'dataPattern' => '',
                    'export' => true,
                ) ,
                'event_end_date' => array(
                    'name' => 'end_date',
                    'type' => CRM_UTILS_TYPE_T_DATE+CRM_UTILS_TYPE_T_TIME,
                    'title' => ts('Event End Date') ,
                    'import' => true,
                    'where' => 'civicrm_event.end_date',
                    'headerPattern' => '/^end|(e(nd\s)?date)$/i',
                    'dataPattern' => '',
                    'export' => true,
                ) ,
                'is_online_registration' => array(
                    'name' => 'is_online_registration',
                    'type' => CRM_UTILS_TYPE_T_BOOLEAN,
                    'title' => ts('Is Online Registration') ,
                ) ,
                'registration_link_text' => array(
                    'name' => 'registration_link_text',
                    'type' => CRM_UTILS_TYPE_T_STRING,
                    'title' => ts('Event Registration Link Text') ,
                    'maxlength' => 255,
                    'size' => CRM_UTILS_TYPE_HUGE,
                ) ,
                'max_participants' => array(
                    'name' => 'max_participants',
                    'type' => CRM_UTILS_TYPE_T_INT,
                    'title' => ts('Max Participants') ,
                ) ,
                'event_full_text' => array(
                    'name' => 'event_full_text',
                    'type' => CRM_UTILS_TYPE_T_TEXT,
                    'title' => ts('Event Information') ,
                ) ,
                'is_monetary' => array(
                    'name' => 'is_monetary',
                    'type' => CRM_UTILS_TYPE_T_BOOLEAN,
                ) ,
                'contribution_type_id' => array(
                    'name' => 'contribution_type_id',
                    'type' => CRM_UTILS_TYPE_T_INT,
                ) ,
                'is_map' => array(
                    'name' => 'is_map',
                    'type' => CRM_UTILS_TYPE_T_BOOLEAN,
                ) ,
                'is_active' => array(
                    'name' => 'is_active',
                    'type' => CRM_UTILS_TYPE_T_BOOLEAN,
                ) ,
            );
        }
        return $GLOBALS['_CRM_EVENT_DAO_EVENT']['_fields'];
    }
    /**
     * returns the names of this table
     *
     * @access public
     * @return string
     */
    function getTableName() 
    {
        return $GLOBALS['_CRM_EVENT_DAO_EVENT']['_tableName'];
    }
    /**
     * returns if this table needs to be logged
     *
     * @access public
     * @return boolean
     */
    function getLog() 
    {
        return $GLOBALS['_CRM_EVENT_DAO_EVENT']['_log'];
    }
    /**
     * returns the list of fields that can be imported
     *
     * @access public
     * return array
     */
    function &import($prefix = false) 
    {
        if (!($GLOBALS['_CRM_EVENT_DAO_EVENT']['_import'])) {
            $GLOBALS['_CRM_EVENT_DAO_EVENT']['_import'] = array();
            $fields = &CRM_Event_DAO_Event::fields();
            foreach($fields as $name => $field) {
                if (CRM_Utils_Array::value('import', $field)) {
                    if ($prefix) {
                        $GLOBALS['_CRM_EVENT_DAO_EVENT']['_import']['event'] = &$fields[$name];
                    } else {
                        $GLOBALS['_CRM_EVENT_DAO_EVENT']['_import'][$name] = &$fields[$name];
                    }
                }
            }
        }
        return $GLOBALS['_CRM_EVENT_DAO_EVENT']['_import'];
    }
    /**
     * returns the list of fields that can be exported
     *
     * @access public
     * return array
     */
    function &export($prefix = false) 
    {
        if (!($GLOBALS['_CRM_EVENT_DAO_EVENT']['_export'])) {
            $GLOBALS['_CRM_EVENT_DAO_EVENT']['_export'] = array();
            $fields = &CRM_Event_DAO_Event::fields();
            foreach($fields as $name => $field) {
                if (CRM_Utils_Array::value('export', $field)) {
                    if ($prefix) {
                        $GLOBALS['_CRM_EVENT_DAO_EVENT']['_export']['event'] = &$fields[$name];
                    } else {
                        $GLOBALS['_CRM_EVENT_DAO_EVENT']['_export'][$name] = &$fields[$name];
                    }
                }
            }
        }
        return $GLOBALS['_CRM_EVENT_DAO_EVENT']['_export'];
    }
}
?>